// **********************************************************************
//
// Copyright (c) 1999
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1999
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_FIXED_H
#define OB_FIXED_H

namespace CORBA
{

class Fixed
{
    //
    // Conversions from other types
    //
    void from_longlong(LongLong);
    void from_ulonglong(ULongLong, Short = 1);
    void from_string(const char*);

public:

#if SIZEOF_LONG == 4
    Fixed(int val = 0) { from_longlong(val); }
    Fixed(unsigned int val) { from_ulonglong(val); }
    Fixed(Long val) { from_longlong(val); }
#else
    Fixed(Long val = 0) { from_longlong(val); }
#endif
    Fixed(ULong val) { from_ulonglong(val); }
    Fixed(LongLong val) { from_longlong(val); }
    Fixed(ULongLong val) { from_ulonglong(val); }
    Fixed(Double);
#if SIZEOF_LONG_DOUBLE != 0
    Fixed(LongDouble);
#endif
    Fixed(const Fixed&);
    Fixed(const char* val) { from_string(val); }
    ~Fixed() { };

    //
    // Array holding the actual value with a precision of up to 64 digits
    // Should be private but this doesn't work with VC++ 6.
    //
    Octet value_[64];

    //
    // The signum
    // Should be private but this doesn't work with VC++ 6.
    //
    Short signum_;

    //
    // The scale
    // Should be private but this doesn't work with VC++ 6.
    //
    Short scale_;

    operator LongLong() const;
    operator LongDouble() const;
    Fixed round(UShort) const;
    Fixed truncate(UShort) const;

    Fixed& operator=(const Fixed&);
    Fixed& operator+=(const Fixed&);
    Fixed& operator-=(const Fixed&);
    Fixed& operator*=(const Fixed&);
    Fixed& operator/=(const Fixed&);

    Fixed& operator++();
    Fixed operator++(int);
    Fixed& operator--();
    Fixed operator--(int);
    Fixed operator+() const;
    Fixed operator-() const;
    Boolean operator!() const;

    UShort fixed_digits() const;
    Short fixed_scale() const;

    //
    // Return absolute value
    //
    Fixed abs() const;

    //
    // Return signum
    //
    Short signum() const;

    //
    // Return scale
    //
    Short scale() const;

    //
    // Set new scale
    //
    Fixed setScale(Short) const;

    //
    // Shift decimal point to the left/right
    //
    Fixed movePointLeft(Short) const;
    Fixed movePointRight(Short) const;

    char* to_string() const;

    Fixed operator+(const Fixed&);
    Fixed operator-(const Fixed&);
    Fixed operator*(const Fixed&);
    Fixed operator/(const Fixed&);
    Boolean operator>(const Fixed&);
    Boolean operator<(const Fixed&);
    Boolean operator>=(const Fixed&);
    Boolean operator<=(const Fixed&);
    Boolean operator==(const Fixed&);
    Boolean operator!=(const Fixed&);
};

} // End of namespace CORBA

OB_STD(istream)& operator>>(OB_STD(istream)&, CORBA::Fixed&);
OB_STD(ostream)& operator<<(OB_STD(ostream)&, const CORBA::Fixed&);

#endif
